Ragequit = LibStub("AceAddon-3.0"):NewAddon("Ragequit","AceConsole-3.0", "AceEvent-3.0")

local version = 1.1
local defaults = {}

function Ragequit:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("RQDB", defaults)

    if (RQDB == nil) then
        RQDB = {}
    end

    local realm = GetRealmName()
    if (RQDB[realm] == nil) then
        RQDB[realm] = {}
    end

    local player = UnitName("player")
    if (RQDB[realm][player] == nil) then
        RQDB[realm][player] = {}
    end

    local faction = UnitFactionGroup("player")
    RQDB[realm][player]["faction"] = faction

    local class = UnitClass("player")
    RQDB[realm][player]["class"] = class
    RQDB[realm][player]["level"] = UnitLevel("player")
    

    Ragequit:RegisterChatCommand('export', 'HandleChatCommand')
    Ragequit:RegisterEvent("TRADE_SKILL_SHOW")
    Ragequit:RegisterEvent("TRADE_SKILL_UPDATE")
    Ragequit:RegisterEvent("CRAFT_UPDATE")
    Ragequit:RegisterEvent("PLAYER_GUILD_UPDATE")
    Ragequit:RegisterEvent("GUILD_ROSTER_UPDATE")
end

function Ragequit:HandleChatCommand(input) 
	Ragequit:ExportCharacters()
end

function Ragequit:ExportCharacters()
    local dialog = StaticPopup_Show("RQEXPORT")
	local editBox = dialog.editBox
	editBox:SetText(Ragequit:Encode(encode(RQDB)))
	editBox:HighlightText()
	editBox:SetFocus()
end

function Ragequit:GetPlayerInfo()
    local realm = GetRealmName()
    local player = UnitName("player")
    return realm, player
end

function Ragequit:Encode(data)
    local Base64Chars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

    return ((data:gsub('.', function(x) 
        local r,Base64Chars='',x:byte()
        for i=8,1,-1 do r=r..(Base64Chars%2^i-Base64Chars%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return Base64Chars:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end

function Ragequit:Decode(data)
    local Base64Chars='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'

    data = string.gsub(data, '[^'..Base64Chars..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(Base64Chars:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
        if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
            return string.char(c)
    end))
end

function Ragequit:TRADE_SKILL_SHOW()
    local realm, player = Ragequit:GetPlayerInfo()

    if (RQDB[realm][player]["tradeskills"] == nil) then
        RQDB[realm][player]["tradeskills"] = {}
    end
end

function Ragequit:PLAYER_GUILD_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    RQDB[realm][player]["guildName"] = guildName
    RQDB[realm][player]["guildRankName"] = guildRankName
    RQDB[realm][player]["guildRankIndex"] = guildRankIndex
end

function Ragequit:GUILD_ROSTER_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()
    local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
    RQDB[realm][player]["guildName"] = guildName
    RQDB[realm][player]["guildRankName"] = guildRankName
    RQDB[realm][player]["guildRankIndex"] = guildRankIndex
end

function FixNil (x)
    if(x== nil)then
        return ''
    else
        return x;
    end
end

StaticPopupDialogs["RQEXPORT"] = {
	text = "CTRL + C nå og lim inn til ragequit.cloud",
	button1 = OKAY,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3,
	hasEditBox = true,
	OnShow = function(self)
		local function HidePopup(self) self:GetParent():Hide() end
		self.editBox:SetScript("OnKeyUp", function(self, key)
            if IsControlKeyDown() and key == "C" then HidePopup(self) end
        end)
	end,
}

function Ragequit:TRADE_SKILL_UPDATE()
    local realm, player = Ragequit:GetPlayerInfo()

    if (RQDB[realm][player]["tradeskills"] == nil) then
        RQDB[realm][player]["tradeskills"] = {}
    end

    local totalRecipes = GetNumTradeSkills()
    local index = GetFirstTradeSkill()

    if (index == 0) then return end

    -- DEFAULT_CHAT_FRAME:AddMessage("RQ: number of skills "..totalRecipies)
    -- DEFAULT_CHAT_FRAME:AddMessage("RQ: first index "..index)

    local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine()
    local data = {}
    data["current"] = currentLevel
    data["max"] = maxLevel
    data["recipes"] = {}

    local name, type
    for i = 1, GetNumTradeSkills() do
        name, cType, _, _, _, _ = GetTradeSkillInfo(i)
        if (name and (cType ~= "header")) then
        	local itemLink = GetTradeSkillItemLink(i)
            table.insert(data["recipes"], {name = name, link = itemLink})
        end
    end

    RQDB[realm][player]["tradeskills"][tradeskillName] = data
end

function Ragequit:CRAFT_UPDATE()
    local craftName = GetCraftName()
    if (craftName == 'Enchanting') then
        local realm, player = Ragequit:GetPlayerInfo()

        if (RQDB[realm][player]["tradeskills"] == nil) then
            RQDB[realm][player]["tradeskills"] = {}
        end

        local totalRecipes = GetNumCrafts()

        if (totalRecipes == 0) then return end

        local name, currentLevel, maxLevel = GetCraftDisplaySkillLine()
        local data = {}
        data["current"] = currentLevel
        data["max"] = maxLevel
        data["recipes"] = {}

        for i = 1, totalRecipes do
            local name, _, cType, _, _, _, _ = GetCraftInfo(i);
            if (name and (cType ~= "header")) then
                local itemLink = GetCraftItemLink(i);
                table.insert(data["recipes"], {name = name, link = itemLink})
            end
        end

        RQDB[realm][player]["tradeskills"]["Enchanting"] = data
    end
end